@@ -9,7 +9,14 @@ pub use perceptron::{
     Algorithm, CWSDefinition, NERDefinition, POSDefinition, PaMode, Perceptron, Trainer,
 };
 #[cfg(feature = "serialization")]
-pub use perceptron::{Codec, Format, ModelSerde, Reader};
+pub use perceptron::{Codec, Format, ModelSerde, Reader, SerdeModel, SerdeCWSModel, SerdePOSModel, SerdeNERModel};
+
+#[cfg(feature = "serialization")]
+pub type CWSModel = SerdeCWSModel;
+#[cfg(feature = "serialization")]
+pub type POSModel = SerdePOSModel;
+#[cfg(feature = "serialization")]
+pub type NERModel = SerdeNERModel;
 
 pub use eisner::eisner;
 pub use entities::{drop_get_entities, get_entities};
