@@ -1,8 +1,10 @@
+use crate::perceptron::definition::CommonDefinePredict;
 use crate::perceptron::GenericItem;
 use crate::perceptron::{
     Definition, TraitFeature, TraitFeatureCompressUtils, TraitFeaturesTrainUtils, TraitParameter,
     TraitParameterStorage, TraitParameterStorageCompressUtils, TraitParameterStorageTrainUtils,
 };
+use anyhow::Result;
 use binary_heap_plus::BinaryHeap;
 use itertools::Itertools;
 use num_traits::NumCast;
@@ -15,7 +17,7 @@ use std::iter::zip;
 use std::mem::swap;
 
 #[cfg_attr(feature = "serialization", derive(Serialize, Deserialize))]
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub enum PaMode<Param>
 where
     Param: TraitParameter,
@@ -74,6 +76,7 @@ where
     Define: Definition,
 {
 }
+
 unsafe impl<Define, Feature, ParamStorage, Param> Sync
     for Perceptron<Define, Feature, ParamStorage, Param>
 where
@@ -198,24 +201,136 @@ where
         }
     }
 
-    pub fn predict(
+    pub fn evaluate(&self, inputs: &[Vec<String>], labels: &[usize]) -> (usize, usize, usize) {
+        let features: Vec<_> = inputs
+            .iter()
+            .map(|f| self.features.get_vector_string(f))
+            .collect();
+        let preds = self.decode(&features);
+        self.definition.evaluate(&preds, labels)
+    }
+}
+
+impl<Define, Feature, ParamStorage, Param> Perceptron<Define, Feature, ParamStorage, Param>
+where
+    Feature: TraitFeature,
+    Param: TraitParameter,
+    ParamStorage: TraitParameterStorage<Param>,
+    Define: Definition + CommonDefinePredict,
+{
+    pub fn predict_with_buffer(
         &self,
         sentence: <Define::RawFeature as GenericItem>::Item,
-    ) -> <Define::Prediction as GenericItem>::Item {
-        let (fragment, features) = self.definition.parse_features(sentence);
+        buffer: &mut Vec<u8>,
+    ) -> Result<<Define::Prediction as GenericItem>::Item> {
+        let (fragment, features) = self
+            .definition
+            .parse_features_with_buffer(&sentence, buffer)?;
         let features: Vec<_> = features
             .iter()
-            .map(|f| self.features.get_vector(f))
+            .map(|f| self.features.get_vector_str(f))
             .collect();
         let preds = self.decode(&features);
 
-        self.definition.predict(&fragment, &preds)
+        Ok(self.definition.predict(&sentence, &fragment, &preds))
+    }
+}
+
+impl<Feature, ParamStorage, Param> Perceptron<POSDefinition, Feature, ParamStorage, Param>
+where
+    Feature: TraitFeature,
+    Param: TraitParameter,
+    ParamStorage: TraitParameterStorage<Param>,
+{
+    pub fn predict(&self, sentence: &[&str]) -> Result<Vec<&str>> {
+        let mut buffer = Vec::with_capacity(sentence.len() * 180);
+        self.predict_with_buffer(sentence, &mut buffer)
+    }
+}
+
+impl<Feature, ParamStorage, Param> Perceptron<NERDefinition, Feature, ParamStorage, Param>
+where
+    Feature: TraitFeature,
+    Param: TraitParameter,
+    ParamStorage: TraitParameterStorage<Param>,
+{
+    pub fn predict(&self, sentence: (&[&str], &[&str])) -> Result<Vec<&str>> {
+        let mut buffer = Vec::with_capacity(sentence.0.len() * 150);
+        self.predict_with_buffer(sentence, &mut buffer)
+    }
+}
+
+use crate::{get_entities, CWSDefinition, NERDefinition, POSDefinition};
+
+impl<Feature, ParamStorage, Param> Perceptron<CWSDefinition, Feature, ParamStorage, Param>
+where
+    Feature: TraitFeature,
+    Param: TraitParameter,
+    ParamStorage: TraitParameterStorage<Param>,
+{
+    pub fn check_feature(&self, feature: &str) -> Option<usize> {
+        self.features.get_with_key(feature)
     }
 
-    pub fn evaluate(&self, inputs: &[Vec<String>], labels: &[usize]) -> (usize, usize, usize) {
-        let features: Vec<_> = inputs.iter().map(|f| self.features.get_vector(f)).collect();
+    pub fn predict<'a>(&self, sentence: &'a str) -> Result<Vec<&'a str>> {
+        let mut buffer = Vec::with_capacity(sentence.len() * 20);
+        self.predict_with_buffer(sentence, &mut buffer)
+    }
+
+    pub fn predict_with_buffer<'a>(
+        &self,
+        sentence: &'a str,
+        buffer: &mut Vec<u8>,
+    ) -> Result<Vec<&'a str>> {
+        let (fragments, features) = self
+            .definition
+            .parse_features_with_buffer(&sentence, buffer)?;
+        let features: Vec<_> = features
+            .iter()
+            .map(|f| self.features.get_vector_str(f))
+            .collect();
         let preds = self.decode(&features);
-        self.definition.evaluate(&preds, labels)
+
+        let preds = self.definition.to_labels(&preds);
+        let preds = get_entities(&preds);
+        Ok(preds
+            .into_iter()
+            .map(|(_, start, end)| {
+                let start = fragments[start];
+                let end = fragments[end + 1];
+                &sentence[start..end]
+            })
+            .collect::<Vec<_>>())
+    }
+}
+
+impl<Feature, ParamStorage, Param> Perceptron<CWSDefinition, Feature, ParamStorage, Param>
+where
+    Feature: TraitFeature + TraitFeaturesTrainUtils,
+    Param: TraitParameter,
+    ParamStorage: TraitParameterStorage<Param> + TraitParameterStorageCompressUtils<Param>,
+{
+    pub fn add_core_rule(&mut self, key: &str, s: Param, b: Param, m: Param, e: Param) {
+        if self.features.get_with_key(key).is_none() {
+            let feature_num = self.parameters.len() / 4;
+            self.features.insert_feature(String::from(key), feature_num);
+
+            // S B M E
+            self.parameters.push(s);
+            self.parameters.push(b);
+            self.parameters.push(m);
+            self.parameters.push(e);
+        }
+    }
+
+    pub fn enable_feature_rule(&mut self, key: &str, feature: &str) {
+        if let Some(index) = self.features.get_with_key(key) {
+            self.features.insert_feature(feature.to_string(), index);
+        }
+    }
+
+    pub fn disable_feature_rule(&mut self, feature: &str) -> Option<usize> {
+        self.features.remove_feature(feature)
     }
 }
 
@@ -238,7 +353,10 @@ where
         mode: &PaMode<Param>,
     ) {
         let label_num = self.definition.label_num();
-        let features: Vec<_> = inputs.iter().map(|f| self.features.get_vector(f)).collect();
+        let features: Vec<_> = inputs
+            .iter()
+            .map(|f| self.features.get_vector_string(f))
+            .collect();
         let preds = self.decode(&features);
 
         if labels.ne(&preds) {
@@ -347,7 +465,10 @@ where
         current: usize,
     ) {
         let label_num = self.definition.label_num();
-        let features: Vec<_> = inputs.iter().map(|f| self.features.get_vector(f)).collect();
+        let features: Vec<_> = inputs
+            .iter()
+            .map(|f| self.features.get_vector_string(f))
+            .collect();
         let preds = self.decode(&features);
 
         if labels.ne(&preds) {
@@ -398,7 +519,10 @@ where
     // 并行 averaged perceptron 算法
     pub fn ap_train_parallel_iter(&mut self, inputs: &[Vec<String>], labels: &[usize]) {
         let label_num = self.definition.label_num();
-        let features: Vec<_> = inputs.iter().map(|f| self.features.get_vector(f)).collect();
+        let features: Vec<_> = inputs
+            .iter()
+            .map(|f| self.features.get_vector_string(f))
+            .collect();
         let preds = self.decode(&features);
 
         if labels.ne(&preds) {
