@@ -2,6 +2,6 @@ mod cws;
 mod ner;
 mod pos;
 
-pub use cws::{PyCWSModel, PyCWSTrainer};
+pub use cws::{CharacterType, PyCWSModel, PyCWSTrainer};
 pub use ner::{PyNERModel, PyNERTrainer};
 pub use pos::{PyPOSModel, PyPOSTrainer};
